#!/usr/bin/lua
local require = require
local Class = Class
local aice, bit, os, io = aice, bit, os, io
local ipairs = ipairs
local pairs = pairs
local printf = printf
local exit = exit
local bit_and = bit_and
local aice_cmd_timeout = aice_cmd_timeout
module("field_upgrade")

FIRMWARE_PROGRAMMING_INIT = 0x00
FIRMWARE_PROGRAMMING_INIT0 = 0xbd
FIRMWARE_PROGRAMMING_END = 0x01
FPGA_PROGRAMMING_INIT = 0x04
FPGA_PROGRAMMING_END = 0x05
DEVICE_IIC = 0x00
DEVICE_PROM = 0xff
MAX_WORD_LEN = 12

function write_firmware(filename)
	local v = aice.set_cmd_verbose(0)
	local t1 = os.time()
	local image = aice.load_image(filename)
	if (image == nil) then
		printf("can not read file %s\n", filename)
		return false
	end

	printf("programming firmware using file %s\n", filename)

	aice.cmd_aice_program_control(FIRMWARE_PROGRAMMING_INIT0)
	if (aice_cmd_timeout) then
		error("AICE version too old and does not support field upgrade\n")
	end
	aice.sleep(2)
	local nwords = 16 * 1024 / 4
	local dot = 0
	local k = 0
	local debug = false
	while (k < nwords) do
		local nwrite = MAX_WORD_LEN
		if ((nwrite + k) > nwords) then
			nwrite = nwords - k
		end
		local wdata = {}
		for i=1,nwrite do
			if (debug) then
				if (k%4==0) then
					printf("%08x", k*4)
				end
			end
			k = k + 1;
			if (k > #image) then
				wdata[i] = 0xffffffff
			else
				wdata[i] = image[k]
			end
			if (k == (0x3ff0/4)+1) then
				wdata[i] = bit_and(wdata[i], 0xffff0000)
			end

			if (debug) then
				printf(" %s", tohex(wdata[i]))
				if (k%4==0) then
					printf("\n")
				end
			end
		end
		io.output():flush()
		aice.cmd_aice_program_write(DEVICE_IIC, wdata);
		if (aice_cmd_timeout) then
			error("upgrade failed. AICE/AICE_MCU could be broken")
		end
		if not debug then
			dot = dot + 1
			printf(".")
			if (dot%32==0) then
				printf("\n")
			end
		end
	end
	printf("\n")
	aice.cmd_aice_program_control(FIRMWARE_PROGRAMMING_END);
	aice.set_cmd_verbose(v)
	local ret
	if (aice_cmd_timeout) then
		printf("firmware programming failed\n");
		ret = false
	else
		printf("firmware programming done\n");
		aice.sleep(1)
		ret = true
	end
	local t2 = os.time()
	local elapsed_time = os.difftime(t2, t1)
	printf("elapsed time = %d:%02d\n", elapsed_time / 60, elapsed_time %60)
	return ret
end

function write_fpga(filename)
	local v = aice.set_cmd_verbose(0)
	local t1 = os.time()

	local image = aice.load_image(filename)
	if (image == nil) then
		printf("can not read file %s\n", filename)
		return false
	end

	printf("programming fpga using file %s\n", filename)

	aice.cmd_aice_program_control(FPGA_PROGRAMMING_INIT)
	if (aice_cmd_timeout) then
		error("AICE version too old and does not support field upgrade\n")
	end
	aice.sleep(2)
	local nwords = 64 * 1024 / 4
	local dot = 0
	local k = 0
	local debug = false
	while (k < nwords) do
		local nwrite = MAX_WORD_LEN
		if ((nwrite + k) > nwords) then
			nwrite = nwords - k
		end
		local wdata = {}
		for i=1,nwrite do
			if (debug) then
				if (k%4==0) then
					printf("%08x", k*4)
				end
			end
			k = k + 1;
			if (k > #image) then
				wdata[i] = 0xffffffff
			else
				wdata[i] = image[k]
			end

			if (debug) then
				printf(" %s", tohex(wdata[i]))
				if (k%4==0) then
					printf("\n")
				end
			end
		end
		io.output():flush()
		aice.cmd_aice_program_write(DEVICE_PROM, wdata);
		if (aice_cmd_timeout) then
			error("upgrade failed. AICE/AICE_MCU could be broken\n")
		end
		if not debug then
			dot = dot + 1
			if (dot%4) == 0 then
				printf(".")
			end
			if (dot%128) == 0 then
				printf("\n")
			end
		end
	end
	printf("\n")
	aice.cmd_aice_program_control(FPGA_PROGRAMMING_END)
	aice.set_cmd_verbose(v)
	local ret
	if (aice_cmd_timeout) then
		printf("fpga prom programming failed\n");
		ret = false
	else
		printf("fpga prom programming done\n");
		aice.sleep(2)
		ret = true
	end
	local t2 = os.time()
	local elapsed_time = os.difftime(t2, t1)
	printf("elapsed time = %d:%02d\n", elapsed_time / 60, elapsed_time %60)
	return ret
end

function read_firmware(filename)
	local v = aice.set_cmd_verbose(0)
	local t1 = os.time()
	aice.cmd_aice_program_control(FIRMWARE_PROGRAMMING_INIT0);
	if (aice_cmd_timeout) then
		error("AICE version too old and does not support field upgrade\n")
	end

	local image = {}
	local nwords = 16 * 1024 / 4
	local dot = 0
	local k = 0
	local debug = false
	while (k < nwords) do
		local nread = MAX_WORD_LEN
		if ((nread + k) > nwords) then
			nread = nwords - k
		end

		local rdata = aice.cmd_aice_program_read(DEVICE_IIC, nread);
		if (rdata == nil or aice_cmd_timeout) then
			os.exit(1)
		end

		if (#rdata ~= nread) then
			printf("ERROR: unexpected # of words read\n");
			exit(1)
		end

		for i,v in ipairs(rdata) do
			if (debug) then
				if (k%4) == 0 then
					printf("[%08x] =", k*4)
				end
			end
			k = k + 1
			image[k] = v

			if (debug) then
				printf(" %s", tohex(v))
				if (k%4) == 0 then
					printf("\n")
				end
			end
		end
		io.flush()
		if not debug then
			dot = dot + 1
			printf(".")
			if (dot % 32) == 0 then
				printf("\n")
			end
		end
	end
	printf("\n")
	aice.sleep(3)
	aice.cmd_aice_program_control(FIRMWARE_PROGRAMMING_END);

	local ret
	if (aice_cmd_timeout) then
		printf("firmware image copying failed\n");
		aice.cmd_clear_timeout()
		ret = false
	else
		aice.save_image(filename, image)
		printf("firmware image copied to file %s\n", filename)
		ret = true
	end
	-- aice.usb_reset()
	aice.set_cmd_verbose(v)
	local t2 = os.time()
	local elapsed_time = os.difftime(t2, t1)
	printf("elapsed time = %d:%02d\n", elapsed_time / 60, elapsed_time %60)
	return ret
end

function read_fpga(filename)
	local v = aice.set_cmd_verbose(0)
	local t1 = os.time()

	aice.cmd_aice_program_control(FPGA_PROGRAMMING_INIT)
	if (aice_cmd_timeout) then
		error("AICE version too old and does not support field upgrade\n")
	end
	aice.sleep(3)
	local image = {}
	local nwords = 128 * 1024 / 4
	local dot = 0
	local k = 0
	local debug = false
	while (k < nwords) do
		local nread = MAX_WORD_LEN
		if ((nread + k) > nwords) then
			nread = nwords - k
		end
		local rdata = {}
		rdata = aice.cmd_aice_program_read(DEVICE_PROM, nread)
		if (aice_cmd_timeout) then
			printf("copy failed at offset 0x%s.", tohex(k*4-4))
			exit(1)
		end

		if (nread ~= #rdata) then
			printf("ERROR: unexpected # of words read\n");
			exit(1)
		end
		for i,v in ipairs(rdata) do
			if (debug) then
				if (k%4) == 0 then
					printf("[%08x] = ", k*4)
				end
			end
			k = k + 1;
			image[k] = v

			if (debug) then
				printf(" %s", tohex(v))
				if (k%4) == 0 then
					printf("\n")
				end
			end
		end
		io.flush()
		if not debug then
			dot = dot + 1
			if (dot%4) == 0 then
				printf(".")
			end
			if (dot%128) == 0 then
				printf("\n")
			end
		end
	end
	printf("\n")
	aice.sleep(3)
	aice.cmd_aice_program_control(FPGA_PROGRAMMING_END)

	local ret
	if (aice_cmd_timeout) then
		ret = false
		printf("fpga prom copied to file %s\n", filename)
		aice.cmd_clear_timeout()
	else
		aice.save_image(filename, image)
		printf("fpga prom copied to file %s\n", filename)
		aice.sleep(3)
		ret = true
	end
	aice.set_cmd_verbose(v)
	local t2 = os.time()
	local elapsed_time = os.difftime(t2, t1)
	printf("elapsed time = %d:%02d\n", elapsed_time / 60, elapsed_time %60)
	return ret
end
